import moment from 'moment/moment';
import { groupByDeep } from 'sling-helpers';

const emptyString = '---';

// get status value by reference id
const formatStatus = (paymentsStatus, id) => {
  const status = paymentsStatus.data
    .find(item => item.id === id);
  return status.name || emptyString;
};

// get movement category value by reference id
const formatMovementCategory = (operationsCategories, id) => {
  const movementCategory = operationsCategories.data
    .find(item => item.id === id);
  return movementCategory.name || emptyString;
};

// format product type by product_id + installments
const formatProductType = (productTypes, item) => {
  if (!item.productTypeId || !item.installments) {
    return emptyString;
  }
  const productType = productTypes.data
    .find(type => type.id === item.productTypeId);
  return `${productType.name} ${item.installments}x`;
};

const getNames = (walletTypeId, walletTypes, productTypes, brands) => {
  const { brandId, productTypeId, name: walletTypeName } = walletTypes.data
    .find(item => item.id === walletTypeId);

  const { name: brandName } = brands.data
    .find(item => item.id === brandId) || { name: 'Stone' };

  const { name: productType } = productTypes.data
    .find(item => item.id === productTypeId);
  return { brandName, productType, walletTypeName };
};

const mergePropsBy = (propName, mergeFn) => (result, current) => {
  const itemsSoFar = result.map(item => item[propName]);

  return (itemsSoFar.includes(current[propName]))
    ? result.map(item => mergeFn(item, current, propName))
    : [...result, current];
};

export const paymentsCalendarModel = (responses) => {
  const [calendarData, paymentsStatus] = responses;
  const groupAndSumPaymentsWithSameSettlementStatus = (groupedDataByDate) => {
    const entries = Object.entries(groupedDataByDate);
    const resultData = {};

    // iterate through each day
    entries.forEach((element) => {
      // get day informations
      const values = element[1];

      // reduce day informations with same settlement_status_id
      resultData[element[0]] = values.reduce((result, value) => {
        const alreadyHasSettlementStatus = result
          .find(item => item.settlement_status_id ===
            value.settlement_status_id);

        if (!alreadyHasSettlementStatus) {
          result.push(value);
        } else {
          alreadyHasSettlementStatus.amount += value.amount;
        }
        return result;
      }, []);
    });

    return resultData;
  };

  // map api data response
  const mapedData = calendarData.data.map(item => ({
    // api date response comes as UTC iso string (2018-07-02T00:00:00.000Z)
    // so the moment formatter must call utc method first
    date: moment(item.referenceDate).utc().format('YYYY-MM-DD'),
    amount: item.amount,
    settlement_status_id: item.settlementStatusId,
    status: formatStatus(paymentsStatus, item.settlementStatusId),
  }));

  const groupedDataByDate = groupByDeep(mapedData, item => item.date);

  return groupAndSumPaymentsWithSameSettlementStatus(groupedDataByDate);
};

const groupAndSumAmountsWithSameWalletTypeId =
  mergePropsBy('walletTypeId', (item, current, propName) =>
    (item[propName] === current[propName]
      ? { ...item, amount: item.amount + current.amount }
      : item));

export const paymentDetailsModel = (responses) => {
  const [
    paymentsDetails,
    walletTypes,
    brands,
    productTypes,
  ] = responses;

  const paymentDetailsMapper = (paymentDetail) => {
    const {
      amount,
      walletTypeId,
      settlementStatusId,
    } = paymentDetail;

    const { brandId } = walletTypes.data.find(item =>
      (item.id === walletTypeId)) || [];

    const {
      productType,
      walletTypeName: name,
      brandName,
    } = getNames(walletTypeId, walletTypes, productTypes, brands) || [];


    return {
      amount,
      walletTypeId,
      settlementStatusId,
      brandId,
      productType,
      name,
      brandName,
    };
  };

  const payments = paymentsDetails.data
    .map(paymentDetailsMapper);

  const groupedBalance = payments
    .reduce(groupAndSumAmountsWithSameWalletTypeId, []);

  const groupedTotal = payments
    .filter(payment => payment.settlementStatusId === 4)
    .reduce(groupAndSumAmountsWithSameWalletTypeId, []);

  const result = groupByDeep(groupedBalance, item =>
    (item.walletTypeId === 1 ? 'other' : 'summary'));

  result.summary = groupByDeep(result.summary, item => item.brandId);

  result.total = [
    {
      name: 'Saldo',
      amount: groupedBalance.reduce((sum, current) => sum + current.amount, 0),
    },
    {
      name: 'Total pago',
      amount: groupedTotal.reduce((sum, current) => sum + current.amount, 0),
    },
  ];

  return [result];
};

const groupAndSumAmountsWithSameMovementCategoryId =
  mergePropsBy('movementCategoryId', (item, current, propName) =>
    (item[propName] === current[propName]
      ? { ...item, amount: item.amount + current.amount }
      : item));

export const paymentDetailsByWalletTypeIdModel = (responses) => {
  const [paymentsDetails, walletTypes, operationsCategories,
    brands, productTypes] = responses;

  const total = { name: 'Saldo final', amount: 0 };
  const result = paymentsDetails.data
    .map((item) => {
      const {
        amount,
        walletTypeId,
        movementCategoryId,
      } = item;

      const byMovementCategoryId =
        movement => (movement.id === movementCategoryId);

      const {
        name: movementCategory,
      } = operationsCategories.data.find(byMovementCategoryId) || [];

      const { brandId } = walletTypes.data.find(wallet =>
        (wallet.id === walletTypeId)) || [];

      const {
        productType,
        walletTypeName,
        brandName,
      } = getNames(walletTypeId, walletTypes, productTypes, brands) || [];

      total.amount += amount;
      return {
        amount,
        brandId,
        productType,
        walletTypeName,
        movementCategory,
        movementCategoryId,
        brandName,
      };
    })
    .reduce(groupAndSumAmountsWithSameMovementCategoryId, []);

  result.total = [total];
  return result;
};

export const paymentOperationsModel = (responses) => {
  const [operationsData, paymentsStatus, operationsCategories,
    productTypes] = responses;
  const result = {
    total: operationsData.total,
  };

  result.data = operationsData.data.map(item => ({
    operation_date: item.operationDate,
    reference_date: item.referenceDate,
    category: formatMovementCategory(
      operationsCategories, item.movementCategoryId),
    gross_amount: item.grossAmount,
    brand_id: item.brandId,
    transaction_key: item.transactionKey,
    net_amount: item.netAmount,
    installment_number: item.installmentNumber,
    settlement_status_id: item.settlementStatusId,
    status: formatStatus(paymentsStatus, item.settlementStatusId),
    product: formatProductType(productTypes, item),
  }));

  return result;
};

export const paymentsWalletsTypesModel = data => data;

export const paymentsStatusModel = data => data;

export const paymentsOperationsCategoriesModel = data => data;
